#!/usr/bin/python
#
# Centralized tool for Fedora configuration utilities
#
# This software may be freely redistributed under the terms of the GNU General Public License.
# 
# Author: Ankit Patel <ankit644@yahoo.com>
# Rewritten By Tejas Dinkar <tejasdinkar AT gmail DOT com>
#
# FIXME: This is an Ugly hack to get the user name of the caller
#        self.user = os.popen("users").read().split()[0]
# FIXME: I don't Like Calling su, but due to xorg conflict, this is my spawn
#	 pid = os.spawnl(os.P_NOWAIT, "/bin/su", "/bin/su","-",
#                             self.user,"-c",command)9448383365
#

import os
os.sys.path.append('/usr/share/system-config-control')
import gtk
import gtk.glade

###############################################
##        Constants Are Defined Here         ##
###############################################
gladepath = "system-config-control.glade"
if not os.path.exists (gladepath):
	gladepath = "/usr/share/system-config-control/" + gladepath

# The tabs of the Notebook
tabs = {
		"software_button": 0,
		"hardware_button": 1,
		"system_button": 2,
		"network_button": 3,
		"server_button": 4,
		"security_button": 5,
		"misc_button": 6,
	}

from applications import applications

def show_message(message):
		'''This Presents a Warning Box with message'''
		dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, message)
		dlg.set_markup(message)
		dlg.set_position(gtk.WIN_POS_CENTER)
		dlg.set_modal(True)
		dlg.run()
		dlg.destroy()

##
## I18N
## 
from rhpl.translate import _ 
import rhpl.translate as translate
domain = 'system-config-control'
translate.textdomain (domain)

class running_application:
	'''This is a Class that will store the Current Running Child Process'''
	def __init__(self, pid = 0, name = ""):
		self.pid = pid
		self.name = name

	def kill(self):
		os.kill(self.pid,9)
	
	def is_dead(self):
		'''This Returns False if the Process is still Alive'''
		
		# This next line gives the process a chance to exit gracefully ;)
		os.waitpid(self.pid,1)
		
		# This next block sends the process the signal 0
		# This should not effect the process at all, but an error will form
		# iff the process does not exist
		try:
			os.kill(self.pid,0)
		except:
			return True
		return False
	
	
class system_config_control:
	
	def __init__(self):
		# save the user whom to run as. This is an ugly hack. FIX it
		self.user = os.popen("users").read().split()[0]
		
		# running is the list of all applications that have been spawned
		self.running = []
		
		self.xml = gtk.glade.XML(gladepath, domain="system-config-control")
		
		self.active_window = None
		
	def run_child(self, command):
		
		# SPAWN The Process, and save the PID of the child
		pid = os.spawnlp(os.P_NOWAIT, command, command)
		
		# add this process to the list of running processes
		# we add the process ID as well as the name of the app
		self.running.append(running_application(pid = pid, name = command))
		
	def update_running(self):
		'''Updates the variable running'''
		for i in self.running:
			if i.is_dead():
				self.running.remove(i)
			
	def on_btn_click(self,button):
		'''This is Called when a Button is Clicked'''
		
		self.update_running()
		
		# get the command name from the application
			
		application = applications[button.name]
		command = application.command
		
		print "Running Command %s" % command

		# This next block checks if the Process is Running
		for i in self.running:
			if i.name == command:
				# The process is alive, make a warning
				show_message(_("This utility is already running somewhere on your desktop."))
				return
		
		# If the Command is installed on this machine, run it, else error
		if application.package_exists():
			self.run_child(command)
		else:
			show_message(_("This utility is not installed on your computer. Please install <span weight=\"bold\">%s</span>") % application.package)
		
	def on_delete_event(self,*args):
		'''This is Called as The Application Dies'''
		self.update_running()
		
		# This Next Block Leaves a Warning if Applications are still Alive
		self.show_alive(action = 'quit')
			
		gtk.main_quit()
	
	def change_window(self,button):
		'''This Changes the Active Button
		button.name will be of the form software_button'''
		self.window_chooser.set_current_page(tabs[button.name])
		
	
	def show_about(self, *args):
		'''Show the About Box'''
		self.about.show()
		
	def show_alive(self, action = 'quit', *args):
		'''Show All Alive Windows. It gives an error message if not asked to quit'''
		self.update_running()

		livelist = []

		for i in self.running:
			livelist.append(i.name)

		if livelist:
			show_message(_("These Applications are Still Running: ")
						+ ", ".join(livelist))
		elif action != 'quit':
			show_message(_("There are no Applications Running"))
	
	def main(self):
		
		self.window = self.xml.get_widget("window_system-config-control")
		self.about = self.xml.get_widget('about_dialog')
		self.about.hide()
		
		self.window_chooser = self.xml.get_widget("window_chooser")
		
		# Next block Connects signals to events
		self.xml.signal_autoconnect ({
		"on_delete_event" : self.on_delete_event,
		"on_btn_click" : self.on_btn_click,
		"change_window" : self.change_window,
		"on_about_activate": self.show_about,
		"show_alive_procs": self.show_alive
		})
		
		gtk.main()
		

if __name__ == '__main__':
	import signal
	signal.signal (signal.SIGINT, signal.SIG_DFL)
	object = system_config_control()
	object.main()
